import 'package:provide/provide.dart';
import 'package:flutter/material.dart';
import 'dart:collection';

/*
 * @auther YuanShuai
 * @created at 2019/4/9 17:55
 * @desc BaseStateManage 状态管理
 */
class DBaseStateM extends ProviderNode {

  DBaseStateM({w,p}) : super(child: w, providers: p, dispose: true);
}

class DBaseN implements Listenable {
  ObserverList<VoidCallback> _dListeners = ObserverList<VoidCallback>();

  bool _debugAssertNotDisposed() {
    assert(() {
      if (_dListeners == null) {
        throw FlutterError(' $runtimeType 你如果执行了dispose()就不能在使用它了');
      }
      return true;
    }());
    return true;
  }

  @protected
  bool get hasListeners {
    assert(_debugAssertNotDisposed());
    return _dListeners.isNotEmpty;
  }

  @override
  void addListener(VoidCallback listener) {
    assert(_debugAssertNotDisposed());
    _dListeners.add(listener);
  }

  @mustCallSuper
  void dispose() {
    assert(_debugAssertNotDisposed());
    _dListeners = null;
  }

  @protected
  void dNotifyChanges() {
    assert(_debugAssertNotDisposed());
    if (_dListeners != null) {
      final List<VoidCallback> localListeners =
          List<VoidCallback>.from(_dListeners);
      for (VoidCallback listener in localListeners) {
        try {
          if (_dListeners.contains(listener)) listener();
        } catch (exception, stack) {
          FlutterError.reportError(FlutterErrorDetails(
              exception: exception,
              stack: stack,
              library: 'foundation library',
              context: '$runtimeType 在调度通知',
              informationCollector: (StringBuffer information) {
                information
                    .writeln('$runtimeType 当前发送通知的是:');
                information.write('  $this');
              }));
        }
      }
    }
  }

  @override
  void removeListener(listener) {
    assert(_debugAssertNotDisposed());
    _dListeners.remove(listener);
  }
}

class ObserverList<T> extends Iterable<T> {
  final List<T> _dlist = <T>[];
  bool _disDirty = false;
  HashSet<T> _dset;

  void add(T item) {
    _disDirty = true;
    _dlist.add(item);
  }

  bool remove(T item) {
    _disDirty = true;
    return _dlist.remove(item);
  }

  @override
  bool contains(Object element) {
    if (_dlist.length < 3) return _dlist.contains(element);

    if (_disDirty) {
      if (_dset == null) {
        _dset = HashSet<T>.from(_dlist);
      } else {
        _dset.clear();
        _dset.addAll(_dlist);
      }
      _disDirty = false;
    }

    return _dset.contains(element);
  }

  @override
  Iterator<T> get iterator => _dlist.iterator;

  @override
  bool get isEmpty => _dlist.isEmpty;

  @override
  bool get isNotEmpty => _dlist.isNotEmpty;
}
